package de.hsel.spm.baudas.web;

import javax.servlet.*;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;

/**
 * Filter implementation class AuthenticationFilter.
 *
 * @author Edgar Schkrob
 */

@WebFilter(urlPatterns = {"/*"})
public class AuthenticationFilter implements Filter{
    private ServletContext context;

    /**
     * Called by the web container to indicate to a filter that it is being placed into service. This filter manages the authentication.
     * @param fConfig This parameter provides access to everything the code needs to work.
     * @throws ServletException Defines a general exception a servlet can throw when it encounters difficulty.
     */

    public void init(FilterConfig fConfig) throws ServletException {
        this.context = fConfig.getServletContext();
        this.context.log("AuthenticationFilter initialized");
    }

    /**
     * The doFilter method of the Filter is called by the container each time a request/response pair is passed through the chain due to a client request for a resource at the end of the chain.
     * @param request This parameter provides access to everything the code needs to work.
     * @param response This parameter provides access to everything the code needs to issue a response.
     * @param chain This parameter allows passing request along the chain of potential handlers until one of them handles the request.
     * @throws ServletException Defines a general exception a servlet can throw when it encounters difficulty.
     * @throws IOException Signals that an I/O exception of some sort has occurred. This class is the general class of exceptions produced by failed or interrupted I/O operations.
     */

    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        HttpServletRequest req = (HttpServletRequest) request;
        HttpServletResponse res = (HttpServletResponse) response;
        HttpSession session = req.getSession(false);
        String url = req.getRequestURI();
        if(url.contains("login") || url.contains("logo") || url.contains("js/")){
            chain.doFilter(request, response);
        } else if (session == null || !((boolean) session.getAttribute("authentication"))) {   //checking whether the session exists and if authentication succeed
            this.context.log("Unauthorized access request");
            res.sendRedirect(req.getContextPath() + "/login.html");
        } else {
            chain.doFilter(request, response);

        }
    }

    public void destroy() {
        //close any resources here
    }
}