package de.hsel.spm.baudas.web;

import org.jetbrains.annotations.NotNull;
import weka.core.converters.CSVLoader;

import javax.servlet.ServletException;
import javax.servlet.annotation.MultipartConfig;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.Part;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;

/**
 * saves uploaded files.
 *
 * @author Johannes Theiner, Julian Hinxlage
 * @version 0.1
 * @since 0.1
 **/

@WebServlet("/upload")
@MultipartConfig
public class Upload extends HttpServlet {

    private static final long serialVersionUID = 14144111845151L;

    /**
     * let weka check if the format is valid.
     *
     * @param stream input stream
     * @return format valid
     */
    private boolean checkFormat(InputStream stream) {
        CSVLoader loader = new CSVLoader();
        try {
            loader.setSource(stream);
            return loader.getDataSet() != null;
        } catch (Exception e) {
            return false;
        }
    }

    @Override
    protected void doPost(@NotNull HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        req.setCharacterEncoding(StandardCharsets.UTF_8.name());

        Part filePart = req.getPart("file");

        if (filePart == null) {
            resp.sendRedirect("error.jsp?code=" + ErrorCode.FILE_NOT_FOUND);
            return;
        }

        if (filePart.getSubmittedFileName() == null) {
            resp.sendRedirect("error.jsp?code=" + ErrorCode.FILE_NOT_FOUND);
            return;
        }

        String fileName = Paths.get(filePart.getSubmittedFileName()).getFileName().toString();

        if (fileName == null) {
            resp.sendRedirect("error.jsp?code=" + ErrorCode.FILE_NOT_FOUND);
            return;
        }

        InputStream inputStream = filePart.getInputStream();

        if (inputStream == null) {
            resp.sendRedirect("error.jsp?code=" + ErrorCode.FILE_NOT_FOUND);
            return;
        }
        if (inputStream.available() == 0) {
            resp.sendRedirect("error.jsp?code=" + ErrorCode.EMPTY_FILE);
            return;
        }

        if (!checkFormat(inputStream)) {
            resp.sendRedirect("error.jsp?code=" + ErrorCode.INVALID_FORMAT);
            return;
        } else {
            inputStream = filePart.getInputStream();
        }

        Path path = Data.add(fileName);
        if (!Files.exists(path)) {
            Files.createFile(path);
        }

        Files.copy(inputStream, path, StandardCopyOption.REPLACE_EXISTING);


        resp.sendRedirect("");
    }
}