package de.hsel.spm.baudas.analysis;

import org.junit.jupiter.api.Test;

import java.io.File;
import java.net.URL;
import java.time.Duration;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTimeout;

/**
 * @author Karsten Eden
 * @version 0.1
 * @since 0.1
 **/
class ShoppingTimesTest {

    @Test
    void test100() {
        URL url = getClass().getClassLoader().getResource("kd100.csv");
        assert url != null;
        ShoppingTimes overview = new ShoppingTimes(new File(url.getFile()));

        AtomicReference<Map<DayHour, Integer>> results = new AtomicReference<>();

        assertTimeout(Duration.ofMillis(1700), () -> results.set(overview.getResult()));

        assertEquals(4, results.get().get(new DayHour("Montag", "10-12 Uhr")));
        assertEquals(9, results.get().get(new DayHour("Samstag", "12-14 Uhr")));
        assertEquals(1, results.get().get(new DayHour("Donnerstag", "14-17 Uhr")));
        assertEquals(3, results.get().get(new DayHour("Mittwoch", ">17 Uhr")));
        assertEquals(6, results.get().get(new DayHour("Dienstag", "<10 Uhr")));
    }

    @Test
    void test1000(){
        URL url = getClass().getClassLoader().getResource("kd1000.csv");
        assert url != null;
        ShoppingTimes overview = new ShoppingTimes(new File(url.getFile()));

        AtomicReference<Map<DayHour, Integer>> results = new AtomicReference<>();

        assertTimeout(Duration.ofMillis(3000), () -> results.set(overview.getResult()));

        assertEquals(37, results.get().get(new DayHour("Montag", "10-12 Uhr")));
        assertEquals(87, results.get().get(new DayHour("Samstag", "12-14 Uhr")));
        assertEquals(23, results.get().get(new DayHour("Donnerstag", "14-17 Uhr")));
        assertEquals(18, results.get().get(new DayHour("Mittwoch", ">17 Uhr")));
        assertEquals(18, results.get().get(new DayHour("Dienstag", "<10 Uhr")));
    }

    @Test
    void test10000() {
        URL url = getClass().getClassLoader().getResource("kd10000.csv");
        assert url != null;
        ShoppingTimes overview = new ShoppingTimes(new File(url.getFile()));

        AtomicReference<Map<DayHour, Integer>> results = new AtomicReference<>();

        assertTimeout(Duration.ofMillis(3000), () -> results.set(overview.getResult()));

        assertEquals(303, results.get().get(new DayHour("Montag", "10-12 Uhr")));
        assertEquals(737, results.get().get(new DayHour("Samstag", "12-14 Uhr")));
        assertEquals(192, results.get().get(new DayHour("Donnerstag", "14-17 Uhr")));
        assertEquals(199, results.get().get(new DayHour("Mittwoch", ">17 Uhr")));
        assertEquals(149, results.get().get(new DayHour("Dienstag", "<10 Uhr")));
    }
}